function [v,u,f]=t_vuf(jdmid,ju,lat);
% T_VUF Computes nodal modulation corrections.
%   [V,U,F]=T_VUF(DATE,JU,LAT) returns the astronomical
%    phase V, the nodal phase modulation U, and the nodal
%    amplitude correction F at a decimal date DATE for the 
%    components specified by index JU (into the CONST structure
%    returned by T_GETCONSTS) at a latitude LAT.
%
%    If LAT is not specified then the Greenwich Phase V is computed
%    with U=0 and F=1. 
%
%    Note that V and U are in 'cycles', not degrees or radians
%    (i.e. multiply by 360 to get degrees)
%
%    If LAT is set to NaN then the nodal corrections are computed for
%    all satellites that do *not* have a "latitude-dependent" correction
%    factor. This is for compatibility with the ways things are done in
%    the xtide package (The latitude-dependent corrections were zeroed
%    out there partly because it was convenient, but this was rationalized
%    by saying that since the forcing occurs at different latitudes then
%    the observations the idea is possibly bogus anyway).

% R. Pawlowicz 8/Nov/99
%     5/Jan/00 - changed to allow for no LAT setting.
%     8/Nov/00 - Added the LAT=NaN option.
 
% Get all the info about constituents

[const,sat,shallow]=t_getconsts;

% calculate astronomical arguments at mid-point of data time series
[astro,ader]=astron(jdmid);


% Phase relative to greenwich (in units of cycles, presumeably)
% (this only returns values when we have doodson#s, i.e. not for the shallow
% water components but these will be computed later)
v=rem( const.doodson*astro+const.semi, 1);

if nargin==3, % If we have a latitude get nodal corrections

  % Apparently the second order terms in the tidal potential go to zero
  % at the equator, but the third order terms do not. Hence when trying to
  % infer the third-order terms from the second order terms the nodal
  % correction factors blow up. In order to prevent this it is assumed that
  % the equatorial forcing is due to second-order forcing off the equator,
  % from about the 5 degree location. latitudes are hence (somewhat arbitrarily)
  % forced to be no closer than 5 deg to the equator.
  if finite(lat) & (abs(lat)<5); lat=sign(lat).*5; end

  slat=sin(pi.*lat./180);
  % satellite amplitude ratio adjustment for latitude 

  rr=sat.amprat;           % no amplitude correction

  if finite(lat),
    j=find(sat.ilatfac==1); % latitude correction for diurnal constituents
    rr(j)=rr(j).*0.36309.*(1.0-5.0.*slat.*slat)./slat;

    j=find(sat.ilatfac==2); % latitude correction for semi-diurnal constituents
    rr(j)=rr(j).*2.59808.*slat;
  else 
    rr(sat.ilatfac>0)=0;
  end;

  % calculate nodal amplitude and phase corrections

  uu=rem( sat.deldood*astro(4:6)+sat.phcorr, 1);

  %%uu=uudbl-round(uudbl);  <_ I think this was wrong. The original
  %                         FORTRAN code is:  IUU=UUDBL
  %                                           UU=UUDBL-IUU
  %                         which is truncation.        


  % Sum up all of the satellite factors for all satellites.

  nsat=length(sat.iconst);
  nfreq=length(const.isat);

  fsum=1+sum(sparse([1:nsat],sat.iconst,rr.*exp(i*2*pi*uu),nsat,nfreq)).';

  f=abs(fsum);
  u=angle(fsum)./(2.*pi);

  % compute amplitude and phase corrections for shallow water constituents 

  for k=find(finite(const.ishallow))',
    ik=const.ishallow(k)+[0:const.nshallow(k)-1];
    f(k)=prod(f(shallow.iname(ik)).^abs(shallow.coef(ik)));
    u(k)=sum( u(shallow.iname(ik)).*shallow.coef(ik) );
    v(k)=sum( v(shallow.iname(ik)).*shallow.coef(ik) );
  end;

  f=f(ju);
  u=u(ju);
  v=v(ju);

else % Astronomical arguments only

  % compute for shallow water constituents 
  for k=find(finite(const.ishallow))',
    ik=const.ishallow(k)+[0:const.nshallow(k)-1];
    v(k)=sum( v(shallow.iname(ik)).*shallow.coef(ik) );
  end;
  v=v(ju);
  f=ones(size(v));
  u=zeros(size(v));
end;


%------------------------------------------------------------
function [astro,ader] = astron(jd)
% [astro,ader] = astron(jd)
% computes the astronomical variables a=[s,h,p,np,pp] (deg) and 
% their time derivatives ader=[ds,dh,dp,dnp,dpp] (deg/day) at the 
% time jd (given in julian days) relative to the reference time 
% origin 12:00 UT Dec 31, 1899.
%
%  INPUT:   jd=matlab date string (UTC)
%
%  OUTPUT:  astro=[tau s h p np pp]
%           ader=[dtau ds dh dp dnp dpp]
%
%       tau = lunar time
%	s = mean longitude of the moon
%	h = mean longitude of the sun
%	p = mean longitude of the solar perigee
%	np = negative of the longitude of the mean ascending node
%	pp = mean longitude of the lunar perigee
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 12/29/98; 1/11/98
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% B. Beardsley 

jd=jd(:)'; % Row vector

% compute number of days from 12:00 UT Dec 31, 1899
d1=jd-datenum(1899,12,31,12,0,0);
d2=0.0001.*d1;

% compute astronomical constants at time d1
coef=[ones(size(jd));d1;d2.*d2;d2.^3];

sc=[  270.434164,13.1763965268,-0.000085,  0.000000039];
hc=[  279.696678, 0.9856473354, 0.00002267,0.000000000];
pc=[  334.329556, 0.1114040803,-0.0007739,-0.00000026];
npc=[-259.183275, 0.0529539222,-0.0001557,-0.00000005];
ppc=[ 281.220833, 0.0000470684, 0.0000339, 0.00000007];

astro=[sc;hc;pc;npc;ppc]*coef./360.0;
astro=astro-floor(astro);% units cycles so just keep fraction of cycle

% compute lunar time tau, based on fractional part of solar day
hh=jd-floor(jd);
tau=hh+astro(2,:)-astro(1,:);
astro=[tau;astro];

% compute rates of change
dcoef=[ones(size(jd));2.0e-8.*d1;3.0e-12.*d1.*d1];

dsc=[13.1763965268,-0.000085  , 0.000000039];
dhc=[ 0.9856473354, 0.00002267, 0.00       ];
dpc=[ 0.1114040803,-0.0007739 ,-0.00000026 ];
dnpc=[0.0529539222,-0.0001557 ,-0.00000005 ];
dppc=[0.0000470684, 0.0000339 , 0.00000007 ];

ader=[dsc;dhc;dpc;dnpc;dppc]*dcoef./(360.0./365.0);
dtau=365.0+ader(2,:)-ader(1,:);
ader=[dtau;ader];

